
#include <_sio_internal.h>
#include <stdio.h>

void *
_sio_rcobj_create(size_t size)
{
	struct _sio_rcobj *nobj;

	if (size < sizeof(*nobj))
		_sio_panic("_sio_rcobj_create: size requested (%d) too small",
		    size);
	nobj = malloc(size);
	if (nobj == NULL)
		return (NULL);

	pthread_mutex_init(&nobj->rco_mutex, pthread_mutexattr_default);
	pthread_cond_init(&nobj->rco_refcnt_cond, pthread_condattr_default);
	nobj->rco_refcnt = 1;

	return (nobj);
}

void
_sio_rcobj_destroy(void *objv)
{
	struct _sio_rcobj *obj = objv;

	if (obj->rco_refcnt != 1)
		_sio_panic("_sio_rcobj_destroy: refcnt (%d) > 1",
		    obj->rco_refcnt);
	pthread_mutex_destroy(&obj->rco_mutex);
	free(obj);
}

void
_sio_rcobj_ref(void *objv)
{
	struct _sio_rcobj *obj = objv;

/*
	fprintf(stderr, "_sio_rcobj_ref\n");
*/

	pthread_mutex_lock(&obj->rco_mutex);
	if (obj->rco_refcnt < 1)
		_sio_panic("_sio_rcobj_ref: refcnt (%d) < 1", obj->rco_refcnt);
	obj->rco_refcnt++;
	pthread_mutex_unlock(&obj->rco_mutex);
}

void
_sio_rcobj_unref(void *objv)
{
	struct _sio_rcobj *obj = objv;

/*
	fprintf(stderr, "_sio_rcobj_unref\n");
*/

	pthread_mutex_lock(&obj->rco_mutex);
	obj->rco_refcnt--;
	if (obj->rco_refcnt < 1)
		_sio_panic("_sio_rcobj_unref: refcnt (%d) < 1", obj->rco_refcnt);
	pthread_cond_broadcast(&obj->rco_refcnt_cond);
	pthread_mutex_unlock(&obj->rco_mutex);
}

unsigned int
_sio_rcobj_refcnt(void *objv)
{
	struct _sio_rcobj *obj = objv;

	return (obj->rco_refcnt);
}

void
_sio_rcobj_waitrefcnt(void *objv, unsigned int refcnt)
{
	struct _sio_rcobj *obj = objv;

	pthread_mutex_lock(&obj->rco_mutex);
	while (obj->rco_refcnt != refcnt) {
		pthread_cond_wait(&obj->rco_refcnt_cond, &obj->rco_mutex);
	};
	pthread_mutex_unlock(&obj->rco_mutex);
}

